/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/* vim: set ts=2 sw=2 expandtab: */
/*
 * Copyright (c) 2009 Drexel University
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Tom Wambold <tom5760@gmail.com>
 */

#include "ns3/core-module.h"
#include "ns3/common-module.h"
#include "ns3/node-module.h"
#include "ns3/helper-module.h"
#include "ns3/mobility-module.h"
#include "ns3/contrib-module.h"

#include "ns3/nhdp.h"

using namespace ns3;

NS_LOG_COMPONENT_DEFINE ("NhdpExample");

int
main (int argc, char *argv[])
{
  LogComponentEnable ("NhdpClientApplication", LOG_LEVEL_ALL);
  LogComponentEnable ("NhdpExample", LOG_LEVEL_ALL);

  CommandLine cmd;
  cmd.Parse (argc, argv);

  NS_LOG_INFO ("Creating nodes...");
  NodeContainer nodes;
  nodes.Create (2);

  NS_LOG_INFO ("Installing internet stack...");
  InternetStackHelper internet;
  internet.Install (nodes);

  NS_LOG_INFO ("Creating wifi channel...");
  YansWifiChannelHelper wifiChannelHelper = YansWifiChannelHelper::Default ();

  NS_LOG_INFO ("Creating wifi phy...");
  YansWifiPhyHelper wifiPhyHelper = YansWifiPhyHelper::Default ();
  wifiPhyHelper.SetChannel (wifiChannelHelper.Create ());

  NS_LOG_INFO ("Creating wifi mac...");
  NqosWifiMacHelper wifiMacHelper = NqosWifiMacHelper::Default ();
  wifiMacHelper.SetType ("ns3::AdhocWifiMac");

  WifiHelper wifiHelper = WifiHelper::Default ();
  wifiHelper.SetStandard (WIFI_PHY_STANDARD_80211a);
  wifiHelper.SetRemoteStationManager ("ns3::ConstantRateWifiManager",
      "DataMode", StringValue ("wifia-54mbs"));

  NS_LOG_INFO ("Creating wifi devices...");
  NetDeviceContainer wifiContainer = wifiHelper.Install (wifiPhyHelper,
      wifiMacHelper, nodes);
  NetDeviceContainer wifiContainer2 = wifiHelper.Install (wifiPhyHelper,
      wifiMacHelper, nodes);

  NS_LOG_INFO ("Assigning IP addresses.");
  Ipv4AddressHelper ipv4 ("10.1.1.0", "255.255.255.0");
  Ipv4InterfaceContainer ipv4container = ipv4.Assign (wifiContainer);

  Ipv4AddressHelper ipv4b ("10.1.2.0", "255.255.255.0");
  Ipv4InterfaceContainer ipv4container2 = ipv4b.Assign (wifiContainer2);

  NS_LOG_INFO ("Creating mobility model...");
  MobilityHelper mobility;
  Ptr<ListPositionAllocator> positionAlloc = CreateObject<ListPositionAllocator> ();
  positionAlloc->Add (Vector (0.0, 0.0, 0.0));
  positionAlloc->Add (Vector (1.0, 0.0, 0.0));
  mobility.SetPositionAllocator (positionAlloc);
  mobility.SetMobilityModel ("ns3::ConstantPositionMobilityModel");
  mobility.Install (nodes);

  NS_LOG_INFO ("Installing applications...");
  NhdpHelper nhdpHelper;
  ApplicationContainer apps = nhdpHelper.Install (nodes);

  /*
  NS_LOG_INFO ("Adding interfaces to NHDP...");
  for (NodeContainer::Iterator iter = nodes.Begin ();
      iter != nodes.End ();
      iter++)
    {
      Ptr<NhdpClient> nhdp = DynamicCast<NhdpClient> ((*iter)->GetApplication (0));
      nhdp->AddLocalInterface(1, true);
    }
  */

  NS_LOG_INFO ("Starting simulation...");
  apps.Start (Seconds (1.0));
  apps.Stop (Seconds (10.0));

  Simulator::Run ();
  Simulator::Destroy ();
}
