/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/* vim: set ts=2 sw=2 expandtab: */
/*
 * Copyright (c) 2009 Drexel University
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Tom Wambold <tom5760@gmail.com>
 */

#ifndef __NHDP_INFO_BASE_H__
#define __NHDP_INFO_BASE_H__

#include <vector>
#include <stdint.h>

#include "ns3/ipv4-interface-address.h"
#include "ns3/nstime.h"

namespace ns3 {

/** Just so I don't need to duplicate all the Ptr methods */
class TupleBase
{
public:
  void Ref (void) const;
  void Unref (void) const;

private:
  mutable uint32_t m_refCount;
};

typedef std::vector<Ipv4InterfaceAddress>::iterator NhdpTupleAddressIterator;
typedef std::vector<Ipv4InterfaceAddress>::const_iterator ConstNhdpTupleAddressIterator;

struct NhdpLocalInterfaceTuple : public TupleBase
{
  std::vector<Ipv4InterfaceAddress> addrList;
  bool isManet;
};

struct NhdpRemovedInterfaceAddressTuple : public TupleBase
{
  Ipv4InterfaceAddress addr;
  Time time;
};

struct NhdpLinkTuple : public TupleBase
{
  std::vector<Ipv4InterfaceAddress> neighborIfaceAddrList;
  Time heardTime;
  Time symTime;
  float quality;
  bool pending;
  bool lost;
  Time time;
};

struct NhdpTwoHopTuple : public TupleBase
{
  std::vector<Ipv4InterfaceAddress> neighborIfaceAddrList;
  Ipv4InterfaceAddress twoHopAddr;
  Time time;
};

struct NhdpNeighborTuple : public TupleBase
{
  std::vector<Ipv4InterfaceAddress> neighborAddrList;
  bool symmetric;
};

struct NhdpLostNeighborTuple : public TupleBase
{
  Ipv4InterfaceAddress address;
  Time time;
};

}

#endif /* __NHDP_INFO_BASE_H__ */
