/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/* vim: set ts=2 sw=2 expandtab: */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/nhdp.h"
#include "ns3/names.h"
#include "nhdp-helper.h"

namespace ns3 {

NhdpHelper::NhdpHelper (void)
{
  m_factory.SetTypeId (NhdpClient::GetTypeId ());
}

void
NhdpHelper::SetAttribute (std::string name, const AttributeValue &value) {
  m_factory.Set (name, value);
}

ApplicationContainer
NhdpHelper::Install (Ptr<Node> node) const
{
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
NhdpHelper::Install (std::string nodeName) const
{
  Ptr<Node> node = Names::Find<Node> (nodeName);
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
NhdpHelper::Install (NodeContainer c) const
{
  ApplicationContainer apps;
  for (NodeContainer::Iterator i = c.Begin (); i != c.End (); ++i)
    {
      apps.Add (InstallPriv (*i));
    }

  return apps;
}

Ptr<Application>
NhdpHelper::InstallPriv (Ptr<Node> node) const
{
  Ptr<Application> app = m_factory.Create<NhdpClient> ();
  node->AddApplication (app);
  
  return app;
}

} /* namespace ns3 */
